@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: Icon.c,v 1.37 2003/06/05 18:47:00 dk Exp $
+ * $Id$
  */
 
 #include "apricot.h"
@@ -64,7 +64,11 @@ produce_mask( Handle self)
       rgbcolor. r = (var-> maskColor >> 16) & 0xFF;
       if ( bpp <= 8)
          color = cm_nearest_color( rgbcolor, var-> palSize, var-> palette);
-   }   
+   } else if ( var-> autoMasking == amMaskIndex) {
+      if ( bpp > 8) return;	 
+      color = var-> maskIndex;
+      bzero( &rgbcolor, sizeof(rgbcolor));
+   }
 
    if ( bpp == imMono) {
       /* mono case simplifies our task */
@@ -75,6 +79,8 @@ produce_mask( Handle self)
          while ( j--) mask[ j] = ~mask[ j];
       }
       var-> palette[color]. r = var-> palette[color]. g = var-> palette[color]. b = 0;
+      if ( color > 0)
+         var-> type &= ~imGrayScale;
       return;
    }
 
@@ -254,15 +260,20 @@ colorFound:;
 
    /* finalize */
    if ( var-> data != area8) free( area8);
-   if ( var-> palSize > color && bpp <= im256)
+   if ( var-> palSize > color && bpp <= im256) {
       var-> palette[ color]. r = var-> palette[ color]. b = var-> palette[ color]. g = 0;
+      if ( color > 0)
+         var-> type &= ~imGrayScale;
+   }
 }
 
 void
 Icon_init( Handle self, HV * profile)
 {
+   dPROFILE;
    inherited init( self, profile);
    my-> set_maskColor( self, pget_i( maskColor));
+   my-> set_maskIndex( self, pget_i( maskIndex));
    my-> set_autoMasking( self, pget_i( autoMasking));
    my-> set_mask( self, pget_sv( mask));
    CORE_INIT_TRANSIENT(Icon);
@@ -312,6 +323,18 @@ Icon_maskColor( Handle self, Bool set, Color color)
    return clInvalid;
 }   
 
+int
+Icon_maskIndex( Handle self, Bool set, int index)
+{
+   if ( !set)
+      return var-> maskIndex;
+   var-> maskIndex = index;
+   if ( is_opt( optInDraw)) return 0;
+   if ( var-> autoMasking == amMaskIndex) 
+      my-> update_change( self);
+   return -1;
+}   
+
 void
 Icon_update_change( Handle self)
 {
@@ -411,6 +434,7 @@ Icon_dup( Handle self)
    memcpy( i-> mask, var-> mask, var-> maskSize);
    i-> autoMasking = var-> autoMasking;
    i-> maskColor   = var-> maskColor;
+   i-> maskIndex   = var-> maskIndex;
    return h;
 }
 
