@@ -49,7 +49,7 @@
 #include <X11/extensions/XShm.h>
 #define USE_MITSHM      1
 #endif
-#if defined(HAVE_X11_XFT_XFT_H) && defined(HAVE_FONTCONFIG_FONTCONFIG_H) && defined(HAVE_X11_EXTENSIONS_XRENDER_H)
+#if defined(HAVE_X11_XFT_XFT_H) && defined(HAVE_FONTCONFIG_FONTCONFIG_H) && defined(HAVE_X11_EXTENSIONS_XRENDER_H) && defined(HAVE_FREETYPE_FREETYPE_H)
 #include <X11/Xft/Xft.h>
 #include <fontconfig/fontconfig.h>
 #  if XFT_MAJOR > 1 && FC_MAJOR > 1
